use rppal::spi::{Bus, Mode, SlaveSelect, Spi};
use std::{thread::sleep, time::Duration};

use crate::packet::FecPacket;

const BUFFER_STATUS_CMD: u8 = 0x00;
const SEND_PACKET_CMD: u8 = 0x01;
const GET_TEMP_CMD: u8 = 0x02;
const SYNC_CMD: u8 = 0x03;
const NOP_CMD: u8 = 0x55;

const PACKET_SPACE: u8 = 0x02;

// Used for talking to an RF4463 via a microcontroller
// The microcontroller takes in packet data over SPI and sends it to the RF4463
pub struct McuRadio {
    spi: Spi,
}

// TODO reset mcu on initialization
impl McuRadio {
    pub fn new(uart: &str) -> anyhow::Result<Self> {
        let spi = Spi::new(Bus::Spi0, SlaveSelect::Ss1, 1_000_000, Mode::Mode0)?;

        Ok(Self { spi })
    }

    pub fn send_packet(&mut self, packet: &FecPacket) -> anyhow::Result<()> {
        // wait until we have packet space
        let mut buf = [0];
        self.spi.transfer(&mut buf, &[NOP_CMD])?;
        while buf != [PACKET_SPACE] {
            sleep(Duration::from_millis(10));

            self.spi.transfer(&mut buf, &[NOP_CMD])?;
        }

        self.spi.write(&[SEND_PACKET_CMD])?;
        self.spi.write(&packet.0)?;

        Ok(())
    }

    pub fn get_temp(&mut self) -> anyhow::Result<f32> {
        // todo need to figure this out

        /*
        self.uart.write_all(&[GET_TEMP_CMD])?;

        let mut buf = [0; 4];
        self.uart.read_exact(&mut buf)?;

        let temp = f32::from_le_bytes(buf);

        Ok(temp) */

        Ok(12.34)
    }

    pub fn sync(&mut self) -> anyhow::Result<()> {
        // todo need to figure this out

        /*

           // clear any pending packets
           let mut tries = 0;
           let mut buf = [0; 100];
           loop {
               match self.uart.read(&mut buf) {
                   Ok(0) => break,
                   Ok(_) => {
                       tries += 1;
                   }
                   Err(e) if matches!(e.kind(), io::ErrorKind::TimedOut) => break,
                   Err(e) => {
                       return Err(e.into());
                   }
               }
           }

           if tries > 0 {
               eprintln!("[WARN] Sync had pending packets(tries={tries})");
           }

           tries = 0;
           let mut buf = [0; 10];
           loop {
               self.uart.write_all(&[SYNC_CMD; 10])?;
               match self.uart.read_exact(&mut buf) {
                   Ok(()) => break,
                   Err(e) if matches!(e.kind(), io::ErrorKind::TimedOut) => {
                       tries += 1;
                   }
                   Err(e) => {
                       return Err(e.into());
                   }
               }
           }

           eprintln!("Synced in {tries} tries");

           Ok(())
        */

        Ok(())
    }

    // sends a bunch of nops over SPI to clear out its DMA buffer
    pub fn flush(&mut self) -> anyhow::Result<()> {
        self.spi.write(&[NOP_CMD; 256])?;

        Ok(())
    }
}
